@@ -1231,6 +1231,9 @@ let rec coq_spfterm c r hyp const bound =
 	 Printf.fprintf c ")"
      | Eq(_) -> failwith("unexpected rewrite step with Eq") (*** symmetry handled by known now ***)
      | Lam((Ar(a,b) as ab),Lam(_,Ap(DB(1,_),DB(0,_)))) ->
+(*** Skip etas - Mar 2012 ***)
+	 coq_spfterm c r1 ((s,h1)::hyp) const bound;
+(***
        (*** Could Skip etas, but don't for now so Coq can type check the result. ***)
        (*** But mark it with a comment ***)
 	 let h2 = h1 ^ "_e" in
@@ -1247,6 +1250,7 @@ let rec coq_spfterm c r hyp const bound =
 	 Printf.fprintf c ") %s (fun %s =>\n" h1 h2;
 	 coq_spfterm c r1 ((s,h2)::hyp) const bound;
 	 Printf.fprintf c "\n) (** eta 2 **)\n"
+***)
      | Lam(a,Lam(_,Ap(Forall(_),Lam(_,(Ap(Ap(Imp,(Ap(DB(0,_),DB(2,_)))),(Ap(DB(0,_),DB(1,_)))) ))) )) -> 
 	 let h2 = get_hyp_name() in
 	 Printf.fprintf c "TRew (";
@@ -1509,7 +1513,8 @@ let ref_coq_spfterm c r =
 	      coq_sterm c con (Variables.make ()) 5001 5000;
 	      Printf.fprintf c " (fun %s => " h1;
 	      coq_spfterm c r ((neg con,h1)::hyp) (!coqsig_const) (Variables.make ());
-	      Printf.fprintf c ")).\nQed.\n"
+	      Printf.fprintf c ")).\nQed.\n";
+	      flush c
 	end
     | None -> raise Not_found
   with
@@ -1517,4 +1522,5 @@ let ref_coq_spfterm c r =
       let hyp = List.fold_left (fun l (s,pt) -> (coqnorm pt,s)::l) [] !coqsig_hyp_trm in
       Printf.fprintf c "exact (";
       coq_spfterm c r hyp (!coqsig_const) (Variables.make ());
-      Printf.fprintf c ").\nQed.\n"
+      Printf.fprintf c ").\nQed.\n";
+      flush c
