@@ -8,306 +8,93 @@ open State
 open Search
 open Proofterm
 
-exception Timeout
 exception InputHelpError of string
 exception InputError of string
 exception UnclassifiedError
 
 let coqfile = ref "";; 
 
-(*** Satallax 2.1 schedules.  One is chosen based on the timeout given. ***)
-(*** 11 modes, 2.0 total time, run when given <= 2 s ***)
+(*** Schedule for determining nontheorems ***)
+let counterstrategy_schedule_1 = [
+("satmode2a",10.0);
+("satmode3b",10.0);
+("satmode2",10.0);
+("satmode3",10.0);
+("mode0d",10.0);
+("satmode1",10.0);
+];;
+
+(*** Satallax 2.4 schedule. ***)
+(*** 61 modes (3 repeated), 260.8 total time ***)
 let strategy_schedule_1 = [
 ("mode205",0.1);
-("mode175",0.2);
-("mode213",0.2);
-("mode233",0.2);
-("mode224",0.3);
-("mode223",0.2);
-("mode194",0.2);
-("mode257",0.2);
-("mode200",0.1);
-("mode204",0.2);
+("mode122c",0.1);
+("mode223m2",0.1);
+("mode222",0.2);
+("mode288",0.2);
+("mode289",0.1);
 ("mode245",0.1);
-];;
-
-(*** 22 modes, 8.0 total time, run when given <= 8 s ***)
-let strategy_schedule_2 = [
-("mode238",0.2);
-("mode223",0.2);
-("mode205",0.5);
-("mode222",0.2);
-("mode198",0.2);
-("mode224",0.6);
-("mode233",0.4);
-("mode175",0.3);
-("mode203",0.4);
-("mode213",0.6);
-("mode257",0.4);
-("mode174",0.6);
-("mode86",0.2);
-("mode104",0.2);
-("mode220",0.2);
-("mode245",0.5);
-("mode163",0.5);
-("mode225",0.3);
-("mode19c",0.3);
-("mode229",0.5);
-("mode253",0.5);
-("mode4a",0.2);
-];;
-
-(*** 35 modes, 17.0 total time, run when given <= 17 s ***)
-let strategy_schedule_3 = [
-("mode194",0.2);
-("mode238",0.2);
-("mode205",0.5);
-("mode222",0.2);
-("mode198",0.2);
-("mode220",0.2);
-("mode175",0.5);
-("mode204",1.0);
-("mode233",0.6);
-("mode4a",0.2);
-("mode252",1.0);
-("mode86",0.2);
-("mode104",0.2);
-("mode245",0.5);
-("mode223",0.9);
-("mode253",0.5);
-("mode163",0.5);
-("mode213",0.9);
-("mode229",0.9);
-("mode19c",0.3);
-("mode226",0.3);
-("mode225",0.8);
-("mode14a",0.1);
-("mode256",0.8);
-("mode108",0.3);
-("mode244",0.5);
-("mode165",0.4);
-("mode188",0.7);
-("mode248",0.3);
-("mode185",0.8);
-("mode236",0.4);
-("mode228",0.4);
-("mode168",0.5);
-("mode249",0.5);
-("mode173",0.5);
-];;
-
-(*** 35 modes, 32.0 total time, run when given <= 32 s ***)
-let strategy_schedule_4 = [
-("mode238",0.2);
-("mode19c",0.3);
-("mode222",0.2);
-("mode233",0.6);
-("mode175",0.5);
-("mode252",0.9);
-("mode198",0.2);
-("mode204",1.4);
-("mode4a",0.2);
-("mode245",0.5);
-("mode104",0.2);
-("mode213",1.6);
-("mode253",0.5);
-("mode224",1.8);
-("mode256",0.8);
-("mode244",0.5);
-("mode173",0.9);
-("mode214",0.2);
-("mode225",1.5);
-("mode163",1.1);
-("mode203",1.3);
-("mode229",0.9);
-("mode236",0.4);
-("mode188",2.0);
-("mode248",0.3);
-("mode223",1.9);
-("mode185",0.8);
-("mode249",0.5);
-("mode86",1.7);
-("mode174",1.4);
-("mode14a",1.2);
-("mode165",1.2);
-("mode8a",1.4);
-("mode228",1.8);
-("mode226",1.1);
-];;
-
-(*** 40 modes, 64.3 total time, run when given <= 67 s ***)
-let strategy_schedule_5 = [
-("mode205",0.1);
-("mode194",0.2);
-("mode222",0.2);
-("mode238",0.2);
-("mode19c",0.3);
-("mode233",0.6);
-("mode175",0.5);
-("mode213",1.6);
-("mode229",0.9);
-("mode4a",0.2);
-("mode104",0.2);
-("mode245",0.5);
-("mode173",0.9);
-("mode253",0.5);
-("mode224",3.8);
-("mode256",0.8);
-("mode244",0.5);
-("mode188",2.0);
-("mode236",0.4);
-("mode204",4.0);
-("mode165",1.2);
-("mode248",0.3);
-("mode14a",1.2);
-("mode163",3.0);
-("mode225",2.7);
-("mode185",0.8);
-("mode86",1.7);
-("mode252",3.2);
-("mode223",3.2);
-("mode228",1.8);
-("mode226",1.9);
-("mode257",2.0);
-("mode249",2.1);
-("mode8a",3.2);
-("mode197",3.5);
-("mode179",1.4);
-("mode219",3.3);
-("mode80",2.2);
-("mode3a",3.3);
-("mode187",3.9);
-];;
-
-(*** 40 modes, 103.9 total time, run when given <= 113 s ***)
-let strategy_schedule_6 = [
-("mode238",0.2);
-("mode19c",0.3);
-("mode222",0.2);
-("mode233",0.6);
-("mode175",0.5);
-("mode198",0.2);
-("mode213",1.6);
-("mode4a",0.2);
-("mode245",0.5);
-("mode104",0.2);
-("mode253",0.5);
-("mode204",4.0);
-("mode236",0.4);
-("mode244",0.5);
-("mode165",1.2);
-("mode256",0.8);
-("mode188",2.0);
-("mode163",1.1);
-("mode185",0.8);
-("mode248",0.3);
-("mode224",6.6);
-("mode173",5.0);
-("mode252",4.7);
-("mode14a",1.2);
-("mode226",1.9);
-("mode225",6.7);
-("mode228",1.8);
-("mode202",6.7);
-("mode257",2.0);
-("mode86",5.1);
-("mode249",2.1);
-("mode207",7.3);
-("mode8a",5.7);
-("mode223",7.4);
-("mode179",4.2);
-("mode80",2.2);
-("mode206",5.3);
-("mode3a",3.3);
-("mode187",3.9);
-("mode171",4.7);
-];;
-
-(*** 38 modes, 167.6 total time, run when given <= 190 s ***)
-let strategy_schedule_7 = [
-("mode238",0.2);
-("mode19c",0.3);
-("mode222",0.2);
-("mode175",0.3);
-("mode220",0.2);
-("mode236",0.4);
-("mode213",1.6);
-("mode244",0.5);
-("mode245",0.5);
-("mode253",0.5);
-("mode4a",0.2);
-("mode188",2.0);
-("mode104",0.2);
-("mode185",0.2);
-("mode252",4.7);
-("mode163",1.1);
-("mode256",0.8);
-("mode248",0.3);
-("mode14a",1.2);
-("mode223",7.4);
-("mode226",0.3);
-("mode250",11.7);
-("mode233",14.6);
-("mode204",16.0);
-("mode228",1.8);
-("mode257",2.0);
-("mode249",2.1);
-("mode86",5.1);
-("mode206",5.3);
-("mode207",7.3);
-("mode8a",5.7);
-("mode251",10.1);
-("mode173",14.5);
-("mode3a",3.3);
-("mode208",11.2);
-("mode179",15.6);
-("mode14",11.5);
-("mode202",6.7);
-];;
-
-(*** 37 modes, 391.8 total time, run when given > 190s, or no timeout is given. ***)
-let strategy_schedule_8 = [
-("mode222",0.6);
-("mode19c",0.5);
-("mode175",0.3);
-("mode236",0.4);
-("mode213",1.6);
-("mode244",0.5);
-("mode245",0.5);
-("mode188",2.0);
-("mode253",0.5);
-("mode4a",0.5);
-("mode104",0.2);
-("mode185",0.2);
-("mode252",4.7);
-("mode163",1.1);
-("mode256",0.8);
-("mode248",0.3);
-("mode223",7.4);
-("mode233",15.0);
-("mode204",28.5);
-("mode228",1.8);
-("mode257",2.0);
-("mode249",2.1);
-("mode206",5.3);
-("mode8a",5.7);
-("mode207",7.3);
-("mode14a",17.0);
-("mode250",40.0);
-("mode251",22.1);
-("mode173",14.5);
-("mode208",11.6);
-("mode179",15.8);
-("mode14",20.2);
-("mode86",30.4);
-("mode202",6.9);
-("mode238",31.7);
-("mode218",23.9);
-("mode165",75.6);
+("mode175",0.2);
+("mode290",0.3);
+("mode252",0.5);
+("mode295",0.2);
+("mode236",0.3);
+("mode284",0.3);
+("mode293",0.6);
+("mode257",0.3);
+("mode185",0.1);
+("mode272",0.1);
+("mode233",1.0);
+("mode19c",0.2);
+("mode261",0.7);
+("mode213",1.3);
+("mode244m5",0.4);
+("mode4a",0.1);
+("mode238",0.1);
+("mode260",0.2);
+("mode205",1.0);
+("mode188",1.2);
+("mode299",0.3);
+("mode93",0.2);
+("mode86",0.8);
+("mode296",0.2);
+("mode204",8.1);
+("mode223m2",2.9);
+("mode267",1.0);
+("mode285",0.9);
+("mode297",0.3);
+("mode249m1",0.3);
+("mode294",0.4);
+("mode276",2.6);
+("mode298",1.1);
+("mode264",3.1);
+("mode286",5.7);
+("mode279",5.2);
+("mode122c",27.3);
+("mode251",4.1);
+("mode250",22.7);
+("mode273",4.3);
+("mode14",6.7);
+("mode283",13.5);
+("mode108",2.5);
+("mode6",2.7);
+("mode282",2.8);
+("mode173",12.8);
+("mode208",3.6);
+("mode170",3.8);
+("mode177",12.6);
+("mode280",4.4);
+("mode223",22.6);
+("mode274",10.9);
+("mode269",17.2);
+("mode252c",11.7);
+("mode165",20.5);
+("mode301",1.0);
+("mode300",10.0);
 ];;
 
 let print_help () =
-  (print_string "Usage: satallax [-[Vvp]] [-verb <int>] [-P <PicoMus>] [-M <modedir>] [-m <modefile>] [-t <timeout in seconds>] <problemfile>";
+  (print_string "Usage: satallax [-[VvN]] [-verb <int>] [-P <PicoMus>] [-M <modedir>] [-m <modefile>] [-t <timeout in seconds>] [-p <pfformat>] <problemfile>";
    print_newline();
    print_string "-M <dir> : Set the directory in which the mode file is stored.";
    print_newline();
@@ -321,6 +108,8 @@ let print_help () =
    print_newline();
    print_string "-v : Verbose";
    print_newline();
+   print_string "-N : Try to determine if the problem is a non-theorem (Satisfiable or CounterSatisfiable)";
+   print_newline();
    print_string "-verb <int> : Verbosity of given level, -verb 0 means silent";
    print_newline();
    print_string "-c [<file>|-] : Create a Coq version of problem, with a proof script if -p is included and proof search succeeds";
@@ -399,7 +188,7 @@ let read_coq_file (f:string) =
 	    seek_in !coqinchannel 0;
 	    List.iter
 	      (fun (x,i) ->
-		if (!verbosity > 20) then Printf.printf "End of Coq fil %d %d\n" i (!j);
+		if (!verbosity > 20) then Printf.printf "End of Coq file %d %d\n" i (!j);
 		match x with
 		| Some g -> if (!verbosity > 20) then g stdout; g !coqoutchannel; seek_in !coqinchannel i; j := i
 		| None -> while (!j < i) do (incr j; let z = input_char !coqinchannel in output_char !coqoutchannel z) done
@@ -454,10 +243,6 @@ let handle_slave_return pstatus =
     if (pstatus >= 10) then exit pstatus else ()
   end;;
 
-(*** set_timer for timeout signal, see http://alan.petitepomme.net/cwn/2006.01.03.html#2 ***)
-let set_timer s =
-  ignore (Unix.setitimer Unix.ITIMER_REAL { Unix.it_interval = 0.0; Unix.it_value = s });;
-
 let problemfile = ref "";;
 
 let process_command_line_args () =
@@ -513,6 +298,12 @@ let process_command_line_args () =
 	    slaveargs := (cf::"-verb"::!slaveargs);
 	    verbosity := (int_of_string cf)
 	  end
+      |	"-N" ->
+	  begin
+	    incr i;
+	    slaveargs := ("-N"::!slaveargs);
+	    nontheorem := true;
+	  end
       | "" -> raise (InputHelpError "Problem processing command line arguments")
       |	"-" -> incr i; problemfile := ""
       | option ->
@@ -538,6 +329,7 @@ let search_main () =
 	try
 	  ignore (List.map load_mode (!mode));
 	  begin
+	    basetypestobool := get_bool_flag "BASETYPESTOPROP";
 	    match (!mkproofterm) with
 	    | Some CoqScript ->
 		begin
@@ -562,8 +354,7 @@ let search_main () =
 	  let s = (get_timeout_default 0.0) in
 	  if (s > 0.0) then
 	    begin
-	      ignore (Sys.signal Sys.sigalrm (Sys.Signal_handle (fun signo -> raise Timeout)));
-	      set_timer s;
+	      if ((!nontheorem) && (get_bool_flag "SPLIT_GLOBAL_DISJUNCTIONS") && (s >= 0.2)) then (set_timer (s *. 0.5); mult_timeout 0.5) else (set_timer s; timeout := Some 0.0);
 	      if (!coq) then coq_init();
 	      if (!coqlocalfile) then read_coq_file (!problemfile) else read_thf_file (!problemfile) (find_read_thf_file (Filename.dirname (!problemfile)));
 	      if ((!coq) && (not (!coq2))) then print_coqsig !coqoutchannel;
@@ -578,6 +369,7 @@ let search_main () =
 	    end
 	with
 	| Unsatisfiable(r) ->
+	    if (not (!nontheorem)) then (*** Some subgoals may have timedout and the last one reported Unsatisfiable ***)
 	    begin
 	      begin
 		match (!mkproofterm,r) with
@@ -633,6 +425,8 @@ let search_main () =
 	      | Some _ -> (if (!verbosity > 0) then (Printf.printf "%% SZS status Theorem\n"; Printf.printf "%% %s \n" (String.concat " " (!mode))); exit 20)
 	      | None -> (if (!verbosity > 0) then (Printf.printf "%% SZS status Unsatisfiable\n"; Printf.printf "%% %s \n" (String.concat " " (!mode))); exit 25)
 	    end
+	    else
+	      raise Timeout (*** if nontheorem was set, then assume we timedout here. ***)
 	| Satisfiable ->
 	    begin
 	      match !conjecture with
@@ -641,26 +435,15 @@ let search_main () =
 	    end
       end
   | [] -> (*** Use a strategy schedule ***)
-      let ltimeout = get_timeout_default 600.0 in (*** If no timeout is given and no mode is given, take a timeout of 10 minutes for these purposes.  The last mode will be run forever. ***)
       let strategy_schedule =
-	begin
-	  if (ltimeout <= 2.0) then
+	if (!nontheorem) then
+	  begin
+	    counterstrategy_schedule_1
+	  end
+	else
+	  begin
 	    strategy_schedule_1
-	  else if (ltimeout <= 8.0) then
-	    strategy_schedule_2
-	  else if (ltimeout <= 17.0) then
-	    strategy_schedule_3
-	  else if (ltimeout <= 32.0) then
-	    strategy_schedule_4
-	  else if (ltimeout <= 67.0) then
-	    strategy_schedule_5
-	  else if (ltimeout <= 113.0) then
-	    strategy_schedule_6
-	  else if (ltimeout <= 190.0) then
-	    strategy_schedule_7
-	  else
-	    strategy_schedule_8
-	end in
+	  end in
       let scheduletime = List.fold_left (fun s1 (x,s2) -> (s1 +. s2)) 0.0 strategy_schedule in (*** total time of schedule ***)
       let timeoutfac = max 1. ((get_timeout_default scheduletime) /. scheduletime) in (*** timeout factor ***)
       let schedulenum = ref (List.length strategy_schedule) in
