@@ -282,7 +282,7 @@ let rec new_search_clause c r =
   il : instantiations for nonvacuous quantifiers
   cg : function for constructing the propositional clauses once the pattern clause has been fully instantiated.
  ***)
-let pattern_clauses : (int * (metatrm option ref list) * (metatrm list) * (int list) * dpair list * metatrm list option * ((bool * stp * metatrm list) list option) * (metatrm list) * (metatrm list) * (metatrm list) * (trm list -> trm list -> trm list -> trm list -> unit)) list ref
+let pattern_clauses : (int * (evar list) * (metatrm list) * (int list) * dpair list * metatrm list option * ((bool * stp * metatrm list) list option) * (metatrm list) * (metatrm list) * (metatrm list) * (trm list -> trm list -> trm list -> trm list -> unit)) list ref
     = ref []
 let pattern_clauses_usable : (int * trm) list ref
     = ref []
@@ -352,7 +352,7 @@ let apply_pattern_clause_2 nalllit ml pl il rewc cg =
 let allstrictp evars p =
   let xl = update_strict [] p in
   try
-    ignore (List.find (fun x -> (not (mem_eq x xl))) evars); (*** If we successfully find such an x, then p does not contain a strict occurrence of all evars ***)
+    ignore (List.find (fun x -> (not (mem_evar x xl))) evars); (*** If we successfully find such an x, then p does not contain a strict occurrence of all evars ***)
     false
   with
   | Not_found -> true (*** p contains a strict occurrence of all evars ***)
@@ -437,13 +437,13 @@ and apply_pattern_clause_1 (nalllit,locevars,unmatched,abslits,dlist,strictlitso
       let evarassoc = ref [] in
       let locevars2 = ref [] in
       List.iter
-	(fun x ->
+	(fun (e,x) ->
 	  match (!x) with
 	  | Some xn -> ()
 	  | None ->
 	      begin
-		let y = copy_evar (!verbosity) x in
-		evarassoc := ((x,y)::(!evarassoc));
+		let y = copy_evar (!verbosity) (e,x) in
+		evarassoc := ((e,y)::(!evarassoc));
 		locevars2 := (y::(!locevars2))
 	      end)
 	locevars;
@@ -463,10 +463,10 @@ and apply_pattern_clause_1 (nalllit,locevars,unmatched,abslits,dlist,strictlitso
 	begin
 	  Printf.printf "*** Matched - locevars2\n";
 	  List.iter
-	    (fun x ->
+	    (fun (e,x) ->
 	      match (!x) with
-	      | Some n -> Printf.printf "%s := %s\n" (string_of_evar x) (metatrm_str n)
-	      | None -> Printf.printf "%s\n" (string_of_evar x)
+	      | Some n -> Printf.printf "%s := %s\n" (string_of_evar (e,x)) (metatrm_str n)
+	      | None -> Printf.printf "%s\n" (string_of_evar (e,x))
 	    )
 	    (!locevars2);
 	  Printf.printf "*** remaining dpairs\n";
@@ -562,13 +562,13 @@ and apply_pattern_clause_eqn ((nalllit,locevars,unmatched,abslits,dlist,strictli
 	let evarassoc = ref [] in
 	let locevars2 = ref [] in
 	List.iter
-	  (fun x ->
+	  (fun (e,x) ->
 	    match (!x) with
 	    | Some xn -> ()
 	    | None ->
 		begin
-		  let y = copy_evar (!verbosity) x in
-		  evarassoc := ((x,y)::(!evarassoc));
+		  let y = copy_evar (!verbosity) (e,x) in
+		  evarassoc := ((e,y)::(!evarassoc));
 		  locevars2 := (y::(!locevars2))
 		end)
 	  locevars;
@@ -588,10 +588,10 @@ and apply_pattern_clause_eqn ((nalllit,locevars,unmatched,abslits,dlist,strictli
 	  begin
 	    Printf.printf "*** Matched - locevars2\n";
 	    List.iter
-	      (fun x ->
+	      (fun (e,x) ->
 		match (!x) with
-		| Some n -> Printf.printf "%s := %s\n" (string_of_evar x) (metatrm_str n)
-		| None -> Printf.printf "%s\n" (string_of_evar x)
+		| Some n -> Printf.printf "%s := %s\n" (string_of_evar (e,x)) (metatrm_str n)
+		| None -> Printf.printf "%s\n" (string_of_evar (e,x))
 	      )
 	      (!locevars2);
 	    Printf.printf "*** remaining dpairs\n";
@@ -689,10 +689,10 @@ and new_pattern_clause_3 c = (*** Process a new clause by trying it with every g
 	    Printf.printf "%d\n" nalllit;
 	    Printf.printf "locevars\n";
 	    List.iter
-	      (fun x ->
+	      (fun (e,x) ->
 		match (!x) with
-		| Some n -> Printf.printf "%s := %s\n" (string_of_evar x) (metatrm_str n)
-		| None -> Printf.printf "%s\n" (string_of_evar x)
+		| Some n -> Printf.printf "%s := %s\n" (string_of_evar (e,x)) (metatrm_str n)
+		| None -> Printf.printf "%s\n" (string_of_evar (e,x))
 	      )
 	      locevars;
 	    Printf.printf "unmatched\n";
@@ -777,14 +777,14 @@ let rec new_pattern_clauses_2 nalllit ml evars strict unmatched abslits remp rem
 	ml;
       Printf.printf "evars\n";
       List.iter
-	(fun x ->
+	(fun (e,x) ->
 	  match (!x) with
-	  | Some n -> Printf.printf "%s := %s\n" (string_of_evar x) (metatrm_str n)
+	  | Some n -> Printf.printf "%s := %s\n" (string_of_evar (e,x)) (metatrm_str n)
 	  | None ->
-	      if (mem_eq x strict) then
-		Printf.printf "%s (strict)\n" (string_of_evar x)
+	      if (mem_evar (e,x) strict) then
+		Printf.printf "%s (strict)\n" (string_of_evar (e,x))
 	      else
-		Printf.printf "%s\n" (string_of_evar x)
+		Printf.printf "%s\n" (string_of_evar (e,x))
 	)
 	evars;
       Printf.printf "unmatched\n";
@@ -891,7 +891,7 @@ let rec new_pattern_clauses_2 nalllit ml evars strict unmatched abslits remp rem
 	| [] -> ()
 	| _ ->
 	    try
-	      let ns = List.find (fun x -> (not (mem_eq x strict))) evars in
+	      let ns = List.find (fun x -> (not (mem_evar x strict))) evars in
 	      if (!verbosity > 40) then
 		Printf.printf "Cannot make a pattern clause, nonstrict evar %s\n" (string_of_evar ns);
 	    with
@@ -1325,6 +1325,7 @@ let generate_type d a =
 
 (*** Assume all types of depth < d are known.  Generate all those of depth d. ***)
 let generate_types d =
+  if (!verbosity > 9) then (Printf.printf "Generate types called with depth %d\n" d; flush stdout);
   if (d = (get_int_flag "ENUM_O")) then generate_type d Prop;
   if (d = (get_int_flag "ENUM_SORT")) then
     (Hashtbl.iter (fun a _ -> generate_type d (Base(a))) name_base);
@@ -1344,6 +1345,7 @@ let generate_types d =
     end
 
 let rec enum_depth d ctx a =
+  if (!verbosity > 9) then Printf.printf "enum_depth %d for type %s\n" d (stp_str a);
   match a with
   | Ar(a1,a2) -> List.map (fun m -> Lam(a1,m)) (enum_depth d (a1::ctx) a2)
   | _ ->
@@ -1356,6 +1358,7 @@ let rec enum_depth d ctx a =
 	end
       else []
 and enum_depth_proj d ctx origctx a i r =
+  if (!verbosity > 9) then Printf.printf "enum_depth_proj %d for type %s i = %d\n" d (stp_str a) i;
   match ctx with
   | (sigma::ctx') ->
       begin
@@ -1370,12 +1373,14 @@ and enum_depth_proj d ctx origctx a i r =
       end
   | _ -> []
 and enum_depth_imit d ctx a r =
+  if (!verbosity > 9) then Printf.printf "enum_depth_imit %d for type %s\n" d (stp_str a);
   (*** Imitations ***)
   (List.iter
      (fun (sigmal,h,pimit) ->
        enum_depth_spine (d - pimit) ctx sigmal h r
      )
      (usable_heads_rtp a));
+  let pneg = get_int_flag "ENUM_NEG" in
   let pimp = get_int_flag "ENUM_IMP" in
   let pfalse = get_int_flag "ENUM_FALSE" in
   let pforall = get_int_flag "ENUM_FORALL" in
@@ -1399,25 +1404,172 @@ and enum_depth_imit d ctx a r =
   done;
   if (a = Prop) then
     begin
-      (*** Imp, False (leaving out Neg here) ***)
+      (*** Neg, Imp, False ***)
+      enum_depth_spine (d - pneg) ctx [Prop] (Lam(Prop,Ap(Ap(Imp,DB(0,Prop)),False))) r;
       enum_depth_spine (d - pimp) ctx [Prop;Prop] (Imp) r;
       enum_depth_spine (d - pfalse) ctx [] (False) r;
     end
 and enum_depth_spine d ctx sigmal h r =
+  if (!verbosity > 9) then Printf.printf "enum_depth_spine %d %s\n" d (trm_str h);
   match sigmal with
   | (a::sigmal') ->
       List.iter
 	(fun m -> enum_depth_spine d ctx sigmal' (Ap(h,m)) r)
 	(enum_depth d ctx a)
-  | [] -> r := (h::!r)
+  | [] ->
+      if (d >= 0) then
+	let m = norm name_def h in
+	if (!verbosity > 9) then Printf.printf "enum_depth_spine finished with %s\n" (trm_str m);
+	r := (m::!r)
+
+(*** Finiteness Assumptions - Apr 2012 ***)
+let stp_finite_elts : (stp,trm list) Hashtbl.t = Hashtbl.create 9
+let stp_finite_sings : (stp,trm list) Hashtbl.t = Hashtbl.create 9
+let stp_finite_axs : (stp,trm list) Hashtbl.t = Hashtbl.create 9
+
+let rec finite_base_type_r a n el sl cov =
+  if (n > 0) then
+    let (ename,e) = get_fresh_name a in
+    let ep = Ap(Ap(Eq(a),DB(0,a)),e) in
+    if (!verbosity > 1) then Printf.printf "New Generating elt %s of finite type %s satisfying\n%s\n" ename (stp_str a) (trm_str ep);
+    finite_base_type_r a (n - 1) (e::el) ((Lam(a,ep))::sl) (disj ep cov)
+  else
+    (el,sl,cov)
+
+let finite_base_type x n =
+  let a = Base x in
+  let (ename,e) = get_fresh_name a in
+  let ep = Ap(Ap(Eq(a),DB(0,a)),e) in
+  if (!verbosity > 1) then Printf.printf "New Generating elt %s of finite type %s satisfying\n%s\n" ename x (trm_str ep);
+  let (el,sl,cov) = finite_base_type_r a (n - 1) [e] [Lam(a,ep)] ep in
+  if (!verbosity > 1) then Printf.printf "Coverage Property of finite type %s\n%s\n" x (trm_str cov);
+  Hashtbl.add stp_finite_elts a el;
+  Hashtbl.add stp_finite_sings a sl;
+  Hashtbl.add stp_finite_axs a [forall a cov]
+
+let rec finite_func_type_1 a domelts codsings fsing =
+  match domelts with
+  | (domelt::domelts) -> finite_func_type_2 a domelt domelts codsings codsings fsing
+  | [] ->
+    begin
+      let felts = Hashtbl.find stp_finite_elts a in
+      let fsings = Hashtbl.find stp_finite_sings a in
+      let faxs = Hashtbl.find stp_finite_axs a in
+      let (ename,e) = get_fresh_name a in
+      if (!verbosity > 1) then Printf.printf "New Generating elt %s of finite type %s satisfying\n%s\n" ename (stp_str a) (trm_str fsing);
+      Hashtbl.add stp_finite_elts a (e::felts);
+      Hashtbl.add stp_finite_sings a (Lam(a,fsing)::fsings);
+      Hashtbl.add stp_finite_axs a ((onlybetanorm (Ap(Lam(a,fsing),e)))::faxs)
+    end
+and finite_func_type_2 a e domelts codsings codsings2 fsing =
+  match codsings with
+  | (codsing::codsings) -> finite_func_type_1 a domelts codsings2 (conj fsing (onlybetanorm (Ap(codsing,Ap(DB(0,a),e))))); finite_func_type_2 a e domelts codsings codsings2 fsing
+  | [] -> ()
+
+let rec finite_func_type_0 a e domelts codsings codsings2 =
+  match codsings with
+  | (codsing::codsings) -> finite_func_type_1 a domelts codsings2 (onlybetanorm (Ap(codsing,Ap(DB(0,a),e)))); finite_func_type_0 a e domelts codsings codsings2
+  | [] -> ()
+
+let finite_func_type a domelts codsings =
+  match domelts with
+  | (domelt::domelts) -> finite_func_type_0 a domelt domelts codsings codsings
+  | [] -> raise (GenericError "Empty Domain Bug")
+
+let rec stp_finite a : unit =
+  if ((not (Hashtbl.mem stp_finite_elts a)) || (List.length (Hashtbl.find stp_finite_elts a) = 0)) then
+    begin
+      if (!verbosity > 1) then Printf.printf "Generating elts of finite type %s\n" (stp_str a);
+      match a with
+      | Ar(a1,a2) ->
+        begin
+          stp_finite a1;
+          stp_finite a2;
+          if (!verbosity > 1) then Printf.printf "Generating funcs %s 1\n" (stp_str a);
+          let domelts = Hashtbl.find stp_finite_elts a1 in
+          if (!verbosity > 1) then Printf.printf "Generating funcs %s 2 %d\n" (stp_str a) (List.length domelts);
+          let codsings = Hashtbl.find stp_finite_sings a2 in
+          if (!verbosity > 1) then Printf.printf "Generating funcs %s 3 %d\n" (stp_str a) (List.length codsings);
+          try
+            Hashtbl.add stp_finite_elts a [];
+            Hashtbl.add stp_finite_sings a [];
+            Hashtbl.add stp_finite_axs a [];
+            finite_func_type a domelts codsings
+          with Timeout -> (*** Destroy partially enumerated type ***)
+            begin
+              Hashtbl.add stp_finite_elts a [];
+              Hashtbl.add stp_finite_sings a [];
+              Hashtbl.add stp_finite_axs a [];
+              raise Timeout
+            end
+          end
+      | Prop ->
+         begin
+           Hashtbl.add stp_finite_elts Prop [False;Ap(Ap(Imp,False),False)];
+           Hashtbl.add stp_finite_sings Prop [Lam(Prop,Ap(Ap(Imp,DB(0,Prop)),False));Lam(Prop,DB(0,Prop))];
+         end
+      | _ -> ()
+    end;;
+
+exception ExistsToChoiceDone
+
+let rec existstochoice1 m p =
+  match m with
+  | Ap(Forall(a),m1) -> (*** to do ***)
+      let q = gen_lam_body a m1 in
+      begin
+	if p then
+	  Ap(Forall(a),Lam(a,existstochoice1 q p))
+	else (*** This is where the action is: Replace (forall x, q[x]) with (q[Eps x:a => ~q[x]]) ***)
+	  subst q 0 (Ap(Choice(a),Lam(a,normneg q)))
+      end
+  | Ap(Ap(Eq(Ar(a1,a2)),m1),m2) ->
+      let m1s = shift m1 0 1 in
+      let m2s = shift m2 0 1 in
+      begin
+	existstochoice1
+	  (norm name_def (forall a1 (eq a2 (ap(m1s,DB(0,a1))) (ap(m2s,DB(0,a1))))))
+	  p
+      end
+  | Ap(Ap(Imp,m1),m2) ->
+      begin
+	try
+	  imp (existstochoice1 m1 (not p)) m2
+	with ExistsToChoiceDone
+	  -> imp m1 (existstochoice1 m2 p)
+      end
+  | _ -> raise ExistsToChoiceDone
+  
+let rec existstochoice m =
+  try
+    existstochoice (existstochoice1 m true)
+  with ExistsToChoiceDone -> m
 
 let already_preprocessed = ref false
 
+let rec preprocess_finite_quants m =
+  match m with
+  | Ap(m1,m2) -> Ap(preprocess_finite_quants m1,preprocess_finite_quants m2)
+  | Lam(a1,m1) -> Lam(a1,preprocess_finite_quants m1)
+  | Forall(a) ->
+    begin
+      stp_finite a;
+      match Hashtbl.find stp_finite_elts a with (*** Replace it with all instances ***)
+      | (e0::elts) -> Lam(Ar(a,Prop),(List.fold_left (fun w e -> conj w (Ap(DB(0,Ar(a,Prop)),e))) (Ap(DB(0,(Ar(a,Prop))),e0)) elts))
+      | _ -> raise (GenericError "Empty type in preprocess_finite_quants")
+    end
+  | _ -> m
+
 let preprocess m =
-  if (get_bool_flag "LEIBEQ_TO_PRIMEQ") then
-    norm name_def (leibeq_to_primeq m)
-  else
-    m
+  let m1 =
+    if (get_bool_flag "LEIBEQ_TO_PRIMEQ") then norm name_def (leibeq_to_primeq m) else m
+  in
+  let m2 =
+    if (get_bool_flag "EXISTSTOCHOICE") then norm name_def (existstochoice m1) else m1
+  in
+  let m3 = if (get_bool_flag "BASETYPESFINITE") then norm name_def (preprocess_finite_quants m2) else m2
+  in
+  m3
 
 let preprocess1 m =
   if (!already_preprocessed) then m else preprocess m
@@ -1510,7 +1662,12 @@ let process_search_option op =
 					    | _ -> ()
 					  end;
 					  choice_rule lh largs;
-					  choice_rule rh rargs
+					  choice_rule rh rargs;
+				      (** Mar 2012 **)
+					  if ((get_bool_flag "HOUNIF1") && (get_bool_flag "HOUNIF1MATE")) then
+					    begin
+					      hounif1 (!verbosity) (to_meta nm) [] [] [] (get_int_flag "HOUNIF1BOUND") false possibly_new_instantiation
+					    end
 					(*** To Do: Handle Special Cases for Categorized Names ***)
 					end
 				end
@@ -1528,7 +1685,12 @@ let process_search_option op =
 				  insertWithPriority pmd (Mating(plit,mlit,pargs,args))
 				    )
 				(Hashtbl.find_all pchoiceatoms a);
-			      choice_rule h args
+			      choice_rule h args;
+				      (** Mar 2012 **)
+			      if ((get_bool_flag "HOUNIF1") && (get_bool_flag "HOUNIF1MATE")) then
+				begin
+				  hounif1 (!verbosity) (to_meta nm) [] [] [] (get_int_flag "HOUNIF1BOUND") false possibly_new_instantiation
+				end
 			    end
 		      | Name(x,_) -> (** negative atom **)
 			  if ((get_bool_flag "FILTER_NEGATM") && (filterp mlit)) then
@@ -1545,7 +1707,12 @@ let process_search_option op =
 					)
 				    (Hashtbl.find_all patoms x)
 				end;
-			      choice_rule h args
+			      choice_rule h args;
+				      (** Mar 2012 **)
+			      if ((get_bool_flag "HOUNIF1") && (get_bool_flag "HOUNIF1MATE")) then
+				begin
+				  hounif1 (!verbosity) (to_meta nm) [] [] [] (get_int_flag "HOUNIF1BOUND") false possibly_new_instantiation
+				end
 			    end
 		      | _ -> raise (GenericError ("Unhandled case (were logical constants normalized?) h:" ^ (trm_str h)))
 		    end
@@ -1558,6 +1725,7 @@ let process_search_option op =
 			let nmlit = (- mlit) in
 			begin
 			  Hashtbl.add univpreds a (nmlit,m1);
+                          if (get_bool_flag "HOUNIF1") then hounif1 (!verbosity) (to_meta m) [] [] [] (get_int_flag "HOUNIF1BOUND") true possibly_new_instantiation; (*** Mar 2012 ***)
 			  let insts = (get_instantiations a) in
 			  begin
 			    match a with
@@ -1578,6 +1746,24 @@ let process_search_option op =
 					if (not (enum_of_started a)) then
 					  let d = get_int_flag "ENUM_START" in
 					  begin
+					    if (not (!enum_started)) then
+					      let d_const = get_int_flag "IMITATE_DELAY" in
+					      let d_def = get_int_flag "IMITATE_DEFN_DELAY" in
+					      begin
+						enum_started := true;
+						Hashtbl.iter
+						  (fun x ((m,sigma),_) ->
+						    let (sigmal,rtp) = argtps_rtp sigma in
+						    if (Hashtbl.mem name_def x) then
+						      begin
+							if (get_bool_flag "IMITATE_DEFNS") then
+							  new_usable_head_rtp rtp sigmal m d_def
+						      end
+						    else
+						      new_usable_head_rtp rtp sigmal m d_const
+							)
+						  name_trm
+					      end;
 					    enum_of_start a;
 					    insertWithPriority d (EnumIterDeep(get_int_flag "ENUM_ITER_DEEP_INIT",a))
 					  end
@@ -1710,6 +1896,11 @@ let process_search_option op =
 					    (fun (otherlit,n1,n2) -> (*** (delayed) confrontation ***)
 					      consider_confrontation a otherlit n1 n2 mlit m1 m2)
 					    (Hashtbl.find_all neqns aname);
+					end;
+				      (** Mar 2012 **)
+				      if ((get_bool_flag "HOUNIF1") && (get_bool_flag "HOUNIF1MATE")) then
+					begin
+					  hounif1 (!verbosity) (to_meta m) [] [] [] (get_int_flag "HOUNIF1BOUND") true possibly_new_instantiation
 					end
 				end
 			    | _ -> raise (GenericError "Eq should have had exactly 2 arguments; there's a bug")
@@ -1727,7 +1918,12 @@ let process_search_option op =
 				  insertWithPriority pmd (Mating(mlit,nlit,args,nargs));
 				)
 				(Hashtbl.find_all nchoiceatoms a);
-			      choice_rule h args
+			      choice_rule h args;
+				      (** Mar 2012 **)
+			      if ((get_bool_flag "HOUNIF1") && (get_bool_flag "HOUNIF1MATE")) then
+				begin
+				  hounif1 (!verbosity) (to_meta m) [] [] [] (get_int_flag "HOUNIF1BOUND") true possibly_new_instantiation
+				end
 			    end
 		      | Name(x,_) -> (** positive atom **)
 			  if ((get_bool_flag "FILTER_POSATM") && (filterp mlit)) then
@@ -1745,7 +1941,12 @@ let process_search_option op =
 				    )
 				    (Hashtbl.find_all natoms x)
 				end;
-			      choice_rule h args (*** This was missing and caused some incompleteness (eg, CHOICE6 in mode59c). - Chad, Jan 25, 2011 ***)
+			      choice_rule h args; (*** This was missing and caused some incompleteness (eg, CHOICE6 in mode59c). - Chad, Jan 25, 2011 ***)
+				      (** Mar 2012 **)
+			      if ((get_bool_flag "HOUNIF1") && (get_bool_flag "HOUNIF1MATE")) then
+				begin
+				  hounif1 (!verbosity) (to_meta m) [] [] [] (get_int_flag "HOUNIF1BOUND") true possibly_new_instantiation
+				end
 			  (*** To Do: Handle Special Cases for Categorized Names ***)
 			    end
 		      | _ -> raise (GenericError ("Unhandled case (were logical constants normalized?) h:" ^ (trm_str h)))
@@ -2069,9 +2270,23 @@ let search_1 b1 b2 =
 	let d = (get_int_flag "FILTER_START") in
 	insertWithPriority d (Filter d)
     end;
+    if (get_bool_flag "BASETYPESFINITE") then
+      begin
+        Hashtbl.iter (fun _ axs ->
+			  (List.iter (fun ax ->
+					  let axlit = get_literal ax in
+					  new_assumption_clause [axlit];
+					  insertWithPriority (get_int_flag "AXIOM_DELAY") (ProcessProp1 ax);
+					  if (!verbosity > 1) then Printf.printf "Putting axiom on queue %s\n" (trm_str ax)
+					  )
+				     axs))
+	stp_finite_axs
+      end;
     search_main ()
   end
 
+let nontheorem_split_timeout : float ref = ref 16.0 (*** default of 16 seconds ***)
+
 let rec split_global_disjunctions b1 b2 b bs sgn cnj =
   match b1 with
     (m::pr) ->
@@ -2153,12 +2368,40 @@ let rec split_global_disjunctions b1 b2 b bs sgn cnj =
 			  | Unsatisfiable(None) ->
 			      reset_search_1 (); incr sgn;
 			      split_global_disjunctions ((normneg m1)::m2::pr) b2 b bs sgn cnj
+			  | Timeout ->
+			      if (!nontheorem) then
+				begin
+				  let s = get_timeout_default 0.0 in
+				  if (s >= 0.2) then
+				    begin
+				      set_timer (s *. 0.5);
+				      mult_timeout 0.5;
+				      reset_search_1 (); incr sgn;
+				      split_global_disjunctions ((normneg m1)::m2::pr) b2 b bs sgn cnj
+				    end
+				  else if (s >= 0.0) then
+				    begin
+				      set_timer s;
+				      timeout := Some 0.0;
+				      reset_search_1 (); incr sgn;
+				      split_global_disjunctions ((normneg m1)::m2::pr) b2 b bs sgn cnj
+				    end
+				  else raise Timeout
+				end
+			      else
+				raise Timeout
 			end
 		    | _ -> raise (GenericError "foo3")
 		  end
 	      | _ ->
 		  split_global_disjunctions pr b2 (m::b) bs sgn cnj
 	    end
+	| Ap(Forall(a),m1) when get_bool_flag "BASETYPESFINITE" -> (*** Apr 2012 - for checking satisfiability in finite models ***)
+	    begin
+              stp_finite a;
+              let elts = Hashtbl.find stp_finite_elts a in (*** Replace it with all instances ***)
+              split_global_disjunctions (List.append (List.map (fun e -> (norm name_def (Ap(m1,e)))) elts) pr) b2 b bs sgn cnj
+	    end
 	| _ ->
 	    let (h,args) = head_spine m in
 	    begin
@@ -2184,6 +2427,28 @@ let rec split_global_disjunctions b1 b2 b bs sgn cnj =
 			  | Unsatisfiable(None) ->
 			      reset_search_1 (); incr sgn;
 			      split_global_disjunctions (m2::pr) b2 b bs sgn cnj
+			  | Timeout ->
+			      if (!nontheorem) then
+				begin
+				  let s = get_timeout_default 0.0 in
+				  if (s >= 0.2) then
+				    begin
+				      set_timer (s *. 0.5);
+				      mult_timeout 0.5;
+				      reset_search_1 (); incr sgn;
+				      split_global_disjunctions (m2::pr) b2 b bs sgn cnj
+				    end
+				  else if (s >= 0.0) then
+				    begin
+				      set_timer s;
+				      timeout := Some 0.0;
+				      reset_search_1 (); incr sgn;
+				      split_global_disjunctions (m2::pr) b2 b bs sgn cnj
+				    end
+				  else raise Timeout
+				end
+			      else
+				raise Timeout
 			end
 		    | _ -> raise (GenericError "foo3")
 		  end
@@ -2207,6 +2472,28 @@ let rec split_global_disjunctions b1 b2 b bs sgn cnj =
 			  | Unsatisfiable(None) ->
 			      reset_search_1 (); incr sgn;
 			      split_global_disjunctions ((normneg m1)::(normneg m2)::pr) b2 b bs sgn cnj
+			  | Timeout ->
+			      if (!nontheorem) then
+				begin
+				  let s = get_timeout_default 0.0 in
+				  if (s >= 0.2) then
+				    begin
+				      set_timer (s *. 0.5);
+				      mult_timeout 0.5;
+				      reset_search_1 (); incr sgn;
+				      split_global_disjunctions ((normneg m1)::(normneg m2)::pr) b2 b bs sgn cnj
+				    end
+				  else if (s >= 0.0) then
+				    begin
+				      set_timer s;
+				      timeout := Some 0.0;
+				      reset_search_1 (); incr sgn;
+				      split_global_disjunctions ((normneg m1)::(normneg m2)::pr) b2 b bs sgn cnj
+				    end
+				  else raise Timeout
+				end
+			      else
+				raise Timeout
 			end
 		    | _ -> raise (GenericError "foo3")
 		  end
@@ -2227,9 +2514,21 @@ let rec split_global_disjunctions b1 b2 b bs sgn cnj =
 		search_1 bs b (*** reversed conjecture and axiom parts to compute axiom parts ***)
 	    end
       end
-	
+
 let search () =
   begin
+    if (not (!nontheorem)) then
+      begin
+        if (get_bool_flag "BASETYPESTOPROP") then
+           raise (GenericError "The flag BASETYPESTOPROP must be false unless trying to determine (Counter)Satisfiablity (-N on the command line).");
+        if (get_bool_flag "BASETYPESFINITE") then
+           raise (GenericError "The flag BASETYPESFINITE must be false unless trying to determine (Counter)Satisfiablity (-N on the command line).");
+      end;
+    (*** Apr 2012: If BASETYPESFINITE is true, then add fresh names for the elements and axiom that says they cover all the elements ***)
+    if (get_bool_flag "BASETYPESFINITE") then
+      begin
+        Hashtbl.iter (fun x _ -> finite_base_type x (get_int_flag "BASETYPESMAXSIZE")) name_base
+      end;
     search_init();
     pattern_clauses_forall_as_lit := get_bool_flag "PATTERN_CLAUSES_FORALL_AS_LIT"; (*** Chad: April 6, 2011 ***)
     pattern_clauses_onlyallstrict := get_bool_flag "PATTERN_CLAUSES_ONLYALLSTRICT"; (*** Chad: April 6, 2011 ***)
@@ -2245,13 +2544,21 @@ let search () =
     else
       ignore (b2 := (!initial_branch)); (*** March 20 2011: In this case it should behave essentially as Satallax 2.0 did. ***)
     if (get_bool_flag "SPLIT_GLOBAL_DISJUNCTIONS") then
-      if (get_bool_flag "PREPROCESS_BEFORE_SPLIT") then
-	let pbr1 = (List.map preprocess1 (!b1)) in
-	let pbr2 = (List.map preprocess1 (!b2)) in
-	already_preprocessed := true;
-	split_global_disjunctions pbr1 pbr2 [] [] (ref 1) true (*** Following Andreas Teucke's suggestion: Preprocess before split - Mar 18 2011 ***)
-      else
-	split_global_disjunctions (!b1) (!b2) [] [] (ref 1) true
+      begin
+	if (!nontheorem) then
+	  begin
+	    match (!timeout) with
+	    | Some tm -> nontheorem_split_timeout := tm
+	    | None -> ()
+	  end;
+	if (get_bool_flag "PREPROCESS_BEFORE_SPLIT") then
+	  let pbr1 = (List.map preprocess1 (!b1)) in
+	  let pbr2 = (List.map preprocess1 (!b2)) in
+	  already_preprocessed := true;
+	  split_global_disjunctions pbr1 pbr2 [] [] (ref 1) true (*** Following Andreas Teucke's suggestion: Preprocess before split - Mar 18 2011 ***)
+	else
+	  split_global_disjunctions (!b1) (!b2) [] [] (ref 1) true
+      end
     else
       search_1 (!b1) (!b2)
   end
