@@ -17,6 +17,7 @@ let slaveargs = ref [Sys.argv.(0)];;
 let mode : string list ref = ref []
 let timeout : float option ref = ref None
 let verbosity : int ref = ref 1 (*** Default to 1; 0 should mean print nothing (-verb 0 command line option) ***)
+let nontheorem : bool ref = ref false (*** March 2012 - Know if you're explicitly trying to determine Satisfiable/CounterSatisfiable ***)
 let coq = ref false
 let coq2 = ref false
 let coqlocalfile = ref false
@@ -59,6 +60,18 @@ let coq_hyp_names : (string,string) Hashtbl.t = Hashtbl.create 100
 let coqknown (x,y) =
   if (!coq2) then y else x
 
+exception Timeout
+
+(*** set_timer for timeout signal, see http://alan.petitepomme.net/cwn/2006.01.03.html#2 ***)
+let set_timer s =
+  ignore (Sys.signal Sys.sigalrm (Sys.Signal_handle (fun signo -> raise Timeout)));
+  ignore (Unix.setitimer Unix.ITIMER_REAL { Unix.it_interval = 0.0; Unix.it_value = s });;
+
+let mult_timeout f =
+  match (!timeout) with
+  | Some tm -> timeout := Some (tm *. f)
+  | None -> ()
+
 let requireSet0a () =
   let a = Base "set" in
   let b = PName "set" in
@@ -75,7 +88,7 @@ let requireSet0a () =
 let required : string ref = ref ""
 
 let require x =
-  if (!verbosity > 0) then Printf.printf "Requiring %s\n" x;
+  if (!verbosity > 5) then Printf.printf "Requiring %s\n" x;
   required := x;
   let f = !coqoutfn1 in
   begin
@@ -746,7 +759,11 @@ let declare_typed_constant (name:string) (role:string) (m:pretrm) =
       POf(PName(x),m) ->
 	begin
 	  match m with
-	    PType ->
+	    PType -> (*** Actually unused. - April 20 2012 ***)
+	      if (Hashtbl.mem name_base x) then raise (Redeclaration x);
+	      if (Hashtbl.mem name_tp x) then raise (Redeclaration x);
+	      declare_base_type x
+	  | PName "$type" -> (*** The case that's used. Added April 20 2012 ***)
 	      if (Hashtbl.mem name_base x) then raise (Redeclaration x);
 	      if (Hashtbl.mem name_tp x) then raise (Redeclaration x);
 	      declare_base_type x
@@ -926,8 +943,6 @@ let new_usable_head_rtp ar sigmal m n = Hashtbl.add usableHeads_rtp ar (sigmal,m
 
 let usable_heads_rtp ar = Hashtbl.find_all usableHeads_rtp ar
 
-(*** Pattern Clauses ***)
-
 (*** search init ***)
 let search_init () =
   (*** Add initial instantiations: true and false for o ***)
