@@ -15,6 +15,7 @@ val slaveargs : string list ref
 val mode : string list ref
 val timeout : float option ref
 val verbosity : int ref
+val nontheorem : bool ref
 val coq : bool ref
 val coq2 : bool ref
 val coqlocalfile : bool ref
@@ -51,6 +52,10 @@ val coq_names : (string,string) Hashtbl.t
 val coq_hyp_names : (string,string) Hashtbl.t
 val coqknown : string * string -> string
 
+exception Timeout
+val set_timer : float -> unit
+val mult_timeout : float -> unit
+
 val required : string ref
 val require : string -> unit
 
