@@ -1198,15 +1198,25 @@ let rec logicnorm m =
   let (n,_) = logicnorm1 m in n
 
 let ideclared = ref false
+let basetypestobool = ref false
 
 (*** conversion of pretrm ***)
 let rec to_stp (m:pretrm) =
   match m with
     PPropTp -> Prop
+  | PIndTp when !basetypestobool -> Prop
+  | PName _ when !basetypestobool -> Prop
   | PIndTp ->
       if (!ideclared) then Base("$i")
-      else (ideclared := true; raise DeclareInd)
-  | PName x -> Base x
+      else
+       begin
+        ideclared := true;
+        raise DeclareInd
+       end
+  | PName x ->
+     begin
+       Base x
+     end
   | PAr(a,b) -> Ar(to_stp a,to_stp b)
   | _ -> raise (GenericSyntaxError ((pretrm_str m) ^ "is not a simple type"))
 
